import os
import subprocess
from enum import Enum
from pathlib import Path

import click
from click.exceptions import ClickException


@click.group()
def cli():
    pass


class Stream(str, Enum):
    KEXP = "kexp"
    KUGS = "kugs"
    CLIS = "clis"


@cli.command("stream")
@click.option(
    "-s", "--stream", "stream", type=click.Choice(Stream), default=Stream.KEXP
)
def play_stream(stream):
    """Play the KEXP stream using mpv."""
    match stream:
        case Stream.KEXP:
            url = "https://kexp-mp3-128.streamguys1.com/kexp128.mp3"
        case Stream.KUGS:
            url = "https://peridot.streamguys1.com:7175/kugs-mp3"
        case Stream.CLIS:
            url = "https://stream2.statsradio.com:8012/stream?=&&___cb=759135934160766"
        case _:
            raise ClickException(f"unrecognized stream: {stream}")
    subprocess.run(["mpv", url])


@cli.command("random")
def play_random():
    """todo."""
    raise NotImplementedError("todo")
    # fd -t file -e mp3 -p -a "fall|winter|spring|summer" $HOME/Music/ | mpv --playlist=- --shuffle --no-video


@cli.command("save")
@click.option("-u", "--url", "url", required=True)
def save_song(url):
    """todo."""
    import yt_dlp

    quarter, year = quarter_year()

    ydl_opts = {
        "format": "mp3/bestaudio/best",
        # ℹ️ See help(yt_dlp.postprocessor) for a list of available Postprocessors and their arguments
        "postprocessors": [
            {  # Extract audio using ffmpeg
                "key": "FFmpegExtractAudio",
                "preferredcodec": "mp3",
            }
        ],
    }

    os.chdir(Path.home() / "Downloads")
    # Download the audio and get the filename
    with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        info_dict = ydl.extract_info(url, download=True)
        _ = ydl.prepare_filename(info_dict)
    if info_dict is None:
        raise ClickException("error downloading")
    filename = info_dict["requested_downloads"][0]["filepath"]

    track = info_dict.get("track")
    artist = info_dict.get("artist")

    parent = Path().home() / "Music" / f"{quarter}_{year}"
    if not parent.exists():
        parent.mkdir(parents=True)
    if track is not None and artist is not None:
        new_filename = f"{artist} - {track}.mp3"
        new_filepath = parent / new_filename

    else:
        new_filepath = parent / filename

    os.rename(filename, new_filepath)


def quarter_year():
    from datetime import datetime

    now = datetime.now()
    year = now.year
    month = now.month
    if month in [12, 1, 2]:
        quarter = "winter"
    elif month in [3, 4, 5]:
        quarter = "spring"
    elif month in [6, 7, 8]:
        quarter = "summer"
    else:
        quarter = "fall"
    return quarter, year


if __name__ == "__main__":
    cli()
